--
-- Sample LUA code for accessing the Windows registry using the WinLUAEx plugin
--

--
-- Generic read function
--
function ReadRegValue(Main, Key, Varname)

-- pass in:
--	the main branch name
--	the key name
--	the variable name

       	local retval, error
	-- create the registry object
	local ro = REGISTRY_CreateObject()
	-- open the key, read only
	local retstring = ro.OpenKey(Main, Key)
	-- if it opened
	if (retstring == nil) then
		-- read the variable value
                retstring, error = ro.Read(Varname)
		-- if the read worked
                if (error == 0) then
			-- save the read value
                        retval = retstring
                else
                      	print("error = "..error..", "..retstring)
                end
		-- close the key and free resources
		ro.CloseKey()
	else
        	print(retstring)
	end
	-- return the read value or nil
	return retval

	-- reval, error, retstring and ro go out of scope and are freed
end

--
-- Generic write function
--
function WriteRegValue(Main, Key, Varname, Varvalue)

-- pass in:
--	the main branch name
--	the key name
--	the variable name
--	the variable value

	-- create the registry object
	local ro = REGISTRY_CreateObject()
	-- create/open the key, read/write
	local retstring = ro.CreateKey(Main, Key)
	-- if it opened
	if (retstring == nil) then
		-- write value to name
        	retstring = ro.Write(Varname, Varvalue)
		-- close the key and free resources
		ro.CloseKey()
 	end
	-- is there was an error
        if (retstring ~= nil) then
                print(retstring)
        end
	-- retstring and ro go out of scope and are freed
end

--
-- List all keys and all values in those keys
--
function ListAll( Main, Key )

-- pass in:
--	the main branch name
--	the key name

	-- create the registry object
	local ro = REGISTRY_CreateObject()
	-- open the key (read only)
	local retstring = ro.OpenKey(Main, Key)
	print("Open "..Main.."\\"..Key)
	if retstring ~= nil then
		print(retstring)
		return
		-- ro and retstring go out of scope and are freed
	end
	-- get a list of subkeys
	local keys
	local error
	keys, error = ro.ListKeys()
	-- did we get a list OK?
	if error == 0 then
		-- for each returned subkey
		while keys.n > 0 do
			-- print the subkey name
			print("Key = "..keys[keys.n])
			-- open the subkey
			retstring = ro.OpenKey(Main, Key.."\\"..keys[keys.n])
			if retstring == nil then
				-- get a list of values
				local values
				values, error = ro.List()
				-- did we get the list OK?
				if error == 0 then
					-- for each value
					while values.n > 0 do
						local value
						value, error = ro.Read(values[values.n])
						-- print the value name and value
						print("\tValueName = "..values[values.n].."\tValue = "..value)
						-- next one
						values.n = values.n - 1
					end
				end
			else
				print(retstring)
			end
			-- next one
			keys.n = keys.n - 1
		end
		-- values goes out of scope and is freed
	end
	-- release the resources
	ro.CloseKey()
	-- ro, retstring, keys and error go out of scope and are freed
end

--
-- Main Code
--

-- global variables
MainBranch = "HKLM" -- HKEY_LOCAL_MACHINE
KeyName = "Software\\Girder3"
SubKeyName = "\\MyStuff"
MyVariable = 100

-- pull the last saved value from the registry
local temp = ReadRegValue(MainBranch, KeyName..SubKeyName, "MyVariable")
-- if we got a value
if (temp ~= nil) then
	-- save temp
	MyVariable = temp
	--
	print("Retrieved "..MyVariable.." from the registry")
else
	--
	print("Write "..MyVariable.." to the registry")
	-- put current value in the registry
        WriteRegValue(MainBranch, KeyName..SubKeyName, "MyVariable", MyVariable)
end

ListAll( MainBranch, KeyName )

-- temp goes out of scope and is freed